;;;; Czasowniki polskie posiadaj dwa tematy: temat czasu teraniejszego oraz
;;;; temat bezokolicznika. Od tematu czasu teraniejszego tworz si:
;;;; czas teraniejszy, czas przyszy prosty, tryb rozkazujcy, imiesowy
;;;; wspczesne - przymiotnikowy i przyswkowy. Od tematu bezokolicznika
;;;; tworz si: bezokolicznik, czas przeszy oraz zaprzeszy, czas przyszy
;;;; zoony, tryb warunkowy, imiesw uprzedni i imiesw bierny. 

;;;; Tematy czasownika czsto s rni i nie sposb ustali jednego z nich
;;;; znajc inny. Dlatego jest bardzo istotnym posiadanie sownika tematw.

;;;; Zalenie od kocwek osobowych czasu teraniejszego, czasowiki dzielimy
;;;; na trzy koniugacji:
;;;; I.  -, -esz, -e, -emy, -ecie, -
;;;;   (pisz, piszesz; myj, myjesz, rysuj, rysujesz)
;;;; II. -, -isz (-ysz), -i (-y), -imy (-ymy), -icie (-ycie), -
;;;;   (widz, widzisz; le, leysz
;;;; III.-am, -asz, -a, -amy, -acie, -
;;;;     -em, -esz, -e, -emy, -ecie, -
;;;;   (czytam, czytasz; umiem, umiesz)
;;;; 
;;;; Oprcz tego mamy pi czasownikw koniugacji archaicznej: by, mie,
;;;; wiedzie, je, da
;;;;
;;;; W I. i II. koniugacjach w czasie teraniejszym czsto wystpuj
;;;; obocznoci.
;;;; Czasowniki twardotematowe I. kon. mog mie dwa tematy liczby
;;;; pojedynczej: temat 1. osoby i temat 2. osoby: bior, bierzesz.
;;;; Temat 1. os. wystpuje w 1. os. lp. oraz 3. os. lm. Temat 2 osoby
;;;; wystpuj we wszystkich innych formach.
;;;;
;;;; Czasowniki, majce w bezokoliczniku przyrostek -owa- zmieniaj go
;;;; na przyrostek -uj-: rysowa - rysuj
;;;;
;;;; Czasowniki II. koniugacji o tematach zakoczonych na -c, -dz i czciowo
;;;; na -sz, -szcz, -d maj takie same obocznoci co i czasowniki I.
;;;; koniugacji.
;;;; 
;;;; Czasowniki III. koniugacji maj w 3. os. lm. temat niecignity,
;;;; czyli z przyrostkiem -aj lub -ej: czytam - czytaj.
;;;;
;;;; Czas przeszy:
;;;; Czas przeszy utwarza si od tematu bezokolicznika. W niektrych wypadkach
;;;; wystpuj obocznoci:
;;;; 1. Jeli temat bezokolicznika koczy si na -e, temat w formie
;;;; mskoosobowej lm. jest taki sam, za w liczbie pojedynczej i w formie
;;;; niemskoosobowej mamy temat na -a-:
;;;;    krzycze - krzycza; krzyczay - krzyczeli
;;;; 
;;;; 2. Jeli temat bezokolicznika koczy si na -, temat czasu przeszego
;;;; jest taki sam w rodzaju mskim, we wszystich za innych formach wystpuje
;;;; -: wzi - wziem - wziam - wzilimy; wziymy
;;;;
;;;; 3. Jeli temat bezokolicznika koczy si na -, -, temat ten wystpuje
;;;; w liczbie mnogiej, osobach 1-3. 3 osoba lp. rodzaju mskiego ma temat
;;;; z odmian zgoski: wie - wiz. Reszta form tworzy si od tematu
;;;; bezokolicznika (praktycznie - od tematu 1. os. lp. czasu teraniwjszego):
;;;; wie - wioz - wiozem, ple - plot - plotem
;;;;
;;;; 4. Czasowniki trze, drze, umrze tworz czas przeszy od tematu
;;;; tar-, dar-, umar-.
;;;;
;;;; Bezokolicznik:
;;;; Wikszo bezokolicznikw ma kocwk -, np.: czyta, chodzi. Niektre
;;;; bezokoliczniki maj na kocu -c: piec, mc, lec.

(defvar tense-table
  '((CT    . "Czas teraniejszy")
    (CPZE  . "Czas przeszy")
    (CZ    . "Czas zapszeszy")
    (CPRZY . "Czas przyszy")))
	   
;; Funkcje analizujce dla poszczeglnych deklinacji/kocwek
;; Kada funkcja zwraca list list:
;;
;; [1]  (temat-czasu-teraniejszego koniugacja czas osoba liczba)
;; FIXME: tryb?

(defmacro verb-set-root (s val)
  `(progn (setf (verb-get-root ,s) ,val) ,s))

(defmacro verb-set-conj (s val)
  `(progn (setf (verb-get-conj ,s) ,val) ,s))

;; Sprawdza zmikszenie tematu
(defun check-lenition (root)
  (if (match-ending root "i")
      (string-concat (remove-ending root 1) "'")
    root))

(defun conj-2-ending (root)
  (or (match-ending root "")
      (match-ending root "dz") ))

;; Sterowniki rozbioru leksycznego
(defvar verb-drivers)
(setq verb-drivers
      (list
       ;; Kocwki czasu przeszego
       (list "aem"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LP
		       (string-concat root "e")
		       (string-concat root "a"))))
       
       (list "em"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LP
		       (string-concat root ""))))

       (list "aam"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LP
		       (string-concat root "e")
		       (string-concat root "a"))))

       (list "am"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LP
		       (string-concat root ""))))

       (list "am"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LP
		       (string-concat root ""))))

       (list "ae"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       2
		       'LP
		       (string-concat root "e")
		       (string-concat root "a"))))
       
       (list "e"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LP
		       (string-concat root ""))))
       
       (list "aa"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       2
		       'LP
		       (string-concat root "e")
		       (string-concat root "a"))))

       (list "a"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       2
		       'LP
		       (string-concat root ""))))

       (list "a"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LP
		       (string-concat root ""))))
       
       (list "a"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
			'CPRZE
			3
			'LP
			(string-concat root "e")
			(string-concat root "a"))))


       (list "a"
	     #'(lambda (word root ending-length)
		 (list root 
		       (list 1 2 3)
		       'CPRZE
		       3
		       'LP
		       (string-concat root ""))))
       
       (list "o"
	     #'(lambda (word root ending-length)
		 (list root 
		       (list 1 2 3)
		       'CPRZE
		       3
		       'LP
		       (string-concat root ""))))
       
       (list "a"
	     #'(lambda (word root ending-length)
		 (list root 
		       (list 1 2 3)
		       'CPRZE
		       3
		       'LP
		       (string-concat root ""))))
       
       (list "o"
	     #'(lambda (word root ending-length)
		 (list root 
		       (list 1 2 3)
		       'CPRZE
		       3
		       'LP
		       (string-concat root ""))))
       
       (list ""
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       3
		       'LP
		       (string-concat root ""))))

       (list "limy"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LM
		       (string-concat root ""))))
       
       (list "limy"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LM
		       (string-concat root ""))))
       
       (list "ymy"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LM
		       (string-concat root ""))))
       
       (list "ymy"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       1
		       'LM
		       (string-concat root ""))))
       
       (list "licie"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       2
		       'LM
		       (string-concat root ""))))
	     
       (list "licie"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       2
		       'LM
		       (string-concat root ""))))
	     
       (list "ycie"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       2
		       'LM
		       (string-concat root ""))))
	     
       (list "ycie"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       2
		       'LM
		       (string-concat root ""))))
	     
       (list "y"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       3
		       'LM
		       (string-concat root ""))))
       (list "li"
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2 3)
		       'CPRZE
		       3
		       'LM
		       (string-concat root ""))))
       
       ;; Kocwki czasu teraniejszego
       (list "uj"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "uj")
		       1 
		       'CT
		       1
		       'LP)))

       (list "ujesz"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "uj")
		       1 
		       'CT
		       2
		       'LP)))

       (list "uje"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "uj")
		       1 
		       'CT
		       3
		       'LP)))
       
       (list "ujemy"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "uj")
		       1 
		       'CT
		       1
		       'LM)))

       (list "ujecie"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "uj")
		       1 
		       'CT
		       2
		       'LM)))

       (list "uj"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "uj")
		       1 
		       'CT
		       3
		       'LM)))
       
       (list ""
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2)
		       'CT
		       1
		       'LP)))
       (list "am"
	     #'(lambda (word root ending-length)
		 (list root
		       3
		       'CT
		       1
		       'LP)))		 

       (list "em"
	     #'(lambda (word root ending-length)
		 (list (check-lenition root)
		       3
		       'CT
		       1
		       'LP)))		 

       (list "esz" ;; 1 kon.
	     #'(lambda (word root ending-length)
		 (list (check-lenition root)
		       (list 1 3)
		       'CT
		       2
		       'LP)))

       (list "isz" ;; 2 kon.
	     #'(lambda (word root ending-length)
		 (list (string-concat root "'")
		       2
		       'CT
		       2
		       'LP)))

       (list "ysz"
	     #'(lambda (word root ending-length)
		 (list root
		       (if (conj-2-ending root)
			   2
			 3)
		       'CT
		       2
		       'LP)))

       (list "asz"
	     #'(lambda (word root ending-length)
		 (list root
		       3
		       'CT
		       2
		       'LP)))

       (list "emy" ; 1 kon.
	     #'(lambda (word root ending-length)
		 (list (check-lenition root)
		       (list 1 3)
		       'CT
		       1
		       'LM)))
       
       (list "imy" ; 2 kon.
	     #'(lambda (word root ending-length)
		 (list (string-concat root "'")
		       2
		       'CT
		       1
		       'LM)))
       
       (list "ymy" ; 2 kon.
	     #'(lambda (word root ending-length)
		 (list root
		       2
		       'CT
		       1
		       'LM)))
       (list "amy" ; 3 kon.
	     #'(lambda (word root ending-length)
		 (list root 
		       3
		       'CT
		       1
		       'LM)))

       (list "ecie" ;; 1 kon.
	     #'(lambda (word root ending-length)
		 (list (check-lenition root)
		       (list 1 3)
		       'CT
		       2
		       'LM)))
       
       (list "icie" ;; 2 kon.
	     #'(lambda (word root ending-length)
		 (list (string-concat root "'")
		       2
		       'CT
		       2
		       'LM)))
       
       (list "ycie" ;; 2 kon.
	     #'(lambda (word root ending-length)
		 (list root 
		       2
		       'CT
		       2
		       'LM)))
       (list "acie" ;; 3 kon.
	     #'(lambda (word root ending-length)
		 (list root 
		       3
		       'CT
		       2
		       'LM)))
       

       (list "e"
	     #'(lambda (word root ending-length)
		 (list (check-lenition root)
		       (list 1 3)
		       'CT
		       3
		       'LP)))

       (list "i"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "'")
		       2
		       'CT
		       3
		       'LP)))
       
       (list "y"
	     #'(lambda (word root ending-length)
		 (list root
		       2
		       'CT
		       3
		       'LP)))
       
       (list "a"
	     #'(lambda (word root ending-length)
		 (list root
		       3
		       'CT
		       3
		       'LP)))

       ;; Czasowniki 3. kon. maj w 3. osobie temat
       ;; niecignity
       (list "iej"
	     ;; Czasowniki mikkotematowe 3. koniugacji
	     #'(lambda (word root ending-length)
		 (list (string-concat root "'")
		       3
		       'CT
		       3
		       'LM)))

       (list "aj"
	     #'(lambda (word root ending-length)
		 (list root
		       3
		       'CT
		       3
		       'LM)))

       (list ""
	     #'(lambda (word root ending-length)
		 (list root
		       (list 1 2)
		       'CT
		       3
		       'LM)))))

;; VD (verb descriptor) jest deskryptorem czasownika o strukturze opisanej wyej (p. [1])
;; Pole koniugacji (nth 1) tego deskryptora moe zawiera list moliwych koniugacji.
;; Funkcja SPLIT-ON-CONJ zwraca list deskryptorw, ktrych pole koniugacji zawiera
;; liczb, innymi sowy ama VD na list:
;;  (split-on-conj '("widz'" '(2 3) CP 1 LP)
;;   => '(("widz'" 2 'CP 1 'LP)
;;        ("widz'" 3 'CP 1 'LP))  
(defun split-on-conj (vd)
  (let ((conj (verb-get-conj vd)))
    (if (listp conj)
	(mapcan
	 #'(lambda (c)
	     (list
	      (append
	       (list (car vd) c)
	       (nthcdr 2 vd))))
	 conj)
      (list vd))))

;; S jest spgosk tematu, po ktrej nastpuje "i", czyli zmikszenie.
;; Funkcja zwraca zmikszon spgosk ("s" => "", itp.)
(defun make-lenited-consonant-ending (s)
  (cond
   ((string-equal s "s") "")
   ((string-equal s "c") "")
   ((string-equal s "z") "")
   ((string-equal s "r") "rz")
   (t
    (string-concat s "ie"))))

;; Prba odgadn bezokolicznik odpowiadajcy deskryptorowi VD. Zwraca anuch.
(defun guess-infinitive (vd)
  (let ((root (verb-get-root vd)))
    (cond
     ((= (verb-get-conj vd) 1)
      (cond
       ((match-ending root "sz")
	(list (string-concat (remove-ending root 2) "sa")))
       ((match-ending root "cz")
	(list (string-concat (remove-ending root 2) "")))
       ((match-ending root "'")
	(list nil))
       (t
	(list (string-concat root "a")))))

     ((= (verb-get-conj vd) 2)
      (cond
       ((match-ending root "'") ; czasowniki mikkotematowe
	(list (string-concat (remove-ending root 1) "ie")))
       ((match-ending root "dz")
	(list (string-concat root "e")
	      (string-concat root "ie")
	      (string-concat root "i")))
       ((match-ending root "dz'")
	(let ((x (remove-ending root 1)))
	  (list (string-concat root "e")
		(string-concat root "ie")
		(string-concat root "i"))))
       ((or (match-ending root "c")
	    (match-ending root "z"))
	(list (string-concat root "e")
	      (string-concat root "i")))
       (t
	; czasowniki twardotematowe
	(list (string-concat root "e")))))
     
     ((= (verb-get-conj vd) 3)
      (cond
       ((match-ending root "'") ; czasowniki mikkotematowe
	(let ((s (substring root (- (length root) 2) (- (length root) 1))))
	  (list (string-concat (remove-ending root 2)
			       (make-lenited-consonant-ending s)
			       ""))))
       (t ; czasowniki twardotematowe
	(list (string-concat root "a"))))))))

;; Wyznacza form bezokolicznika, odpowiadajc deskryptorowi VD.
;; Jeli jest moliwym ustalenie formy bezokolicznika, forma ta zamienia
;; pole tematu, czyli
;;  (deduce-infinitive '("rysuj" 1 CT 1 LP)) => '("rysowa" 1 CT 1 LP)
;; Inaczej, moliwe formy bezokolicznika s dodawane do listy:
;;  (deduce-infinitive '("wo" 1 CT 1 CP)) => ("wo" 1 CT 1 CP "woa")
(defun deduce-infinitive (vd)
  (let* ((root (verb-get-root vd))
	 (inf (dict-verb-present-lookup root (verb-get-conj vd))))
    (cond
     (inf
      (verb-set-root vd inf))
     ((match-ending root "uj")
      ;; Czasowniki majce w bezokoliczniku przyrostek -owa- zmieniaj go
      ;; na przyrostek -uj-
      ;; Ale czasowniki 1. kon. o temacie na spgosk mikk lub
      ;; stwardnia tej obocznoci nie posiadaj
      (append vd (list (string-concat (remove-ending root 2)
				      "owa"))))
     ((and (= (verb-get-conj vd) 1) (match-ending root "j"))
      (append vd (list (string-concat (remove-ending root 1) ""))))
     
     (t
      (append vd (guess-infinitive vd))))))

;; I-LIST jest list bezokolicznikw. CONJ jest numerem koniugacji.
;; Funkcja zwraca lis bezokolicznikw o podanej koniugacji, odnalezionych w
;; sowniku VERB-PRESENT (patrz niej)
(defun check-infinitives (i-list conj)
  (mapcan
   #'(lambda (x)
       (let ((p (dict-verb-infinitive-lookup x conj)))
	 (when (and (not p)
		    (match-ending x "")
		    (consonant-ending (remove-ending x 1)))
	   (setq p (cond
		    ((match-ending x "g")
		     (dict-verb-infinitive-lookup
		      (string-concat (remove-ending x 2)
				     "e")
		      conj))
		    ((match-ending x "szed")
		     (dict-verb-infinitive-lookup
		      (string-concat (remove-ending x 5)
				     "j")
		      conj))
		    (t
		     (setq p (dict-verb-infinitive-lookup
			      (string-concat (remove-ending x 1)
					     "n")
			      conj))))))
	 (when p
	   (list p))))
   i-list))

;; VERB jest form czasownika (acuch).
;; Funkcja zwraca list *moliwych* descryptorw (struktur patrz w [1]) tego
;; czasownika.
(defun analyze-verb (verb)
  (when (> (length verb) 1)
    (let ((d (dict-verb-search-infinitive verb)))
      (if d
	  (list d)
	(let ((vd (dict-verb-individual-lookup verb)))
	  (if vd
	      (list vd)
	    (let* ((result (mapcan
			    #'(lambda (x)
				(if (eq (verb-get-tense x) 'CT)
				    ;; Czas teraniejszy
				    (let ((inf (deduce-infinitive x)))
				      (unless (and (= (length inf) 6) (not (nth 5 inf)))
					(list inf)))
				  ;; Czas przeszy
				  (let ((inf-list (check-infinitives
						   (nthcdr 5 x) (verb-get-conj x))))
				    (if inf-list
					(cond
					 ((= (length inf-list) 1)
					  (list 
					   (list (car inf-list)
						 (nth 1 x)
						 (nth 2 x)
						 (nth 3 x)
						 (nth 4 x))))
					 (t
					  (rplacd (nthcdr 4 x) inf-list))
					 (list x))
				      (list x)))))
			    
			    (mapcan
			     #'(lambda (x)
				 (split-on-conj x))
			     (analyze-word verb verb-drivers))))
		   (resolved (mapcan
			      #'(lambda (x)
				  (when (= (length x) 5)
				    (list x)))
			      result)))
	      
	      (if resolved
		  resolved
		result))))))))
  
;; Sownik. Format:
;;   (list temat-czasu-teraniejszego bezokolicznik koniugacja)
(defvar verb-present nil)

;; Sownik czasownikw koniugacji archaicznej. 
(defvar verb-individual nil)
(setq verb-individual
      '(("jestem"    "by" 0 CT 1 LP)
	("jeste"    "by" 0 CT 2 LP)
	("jest"      "by" 0 CT 3 LP)
	("jestemy"  "by" 0 CT 1 LM)
	("jestecie" "by" 0 CT 2 LM)
	("s"        "by" 0 CT 3 LM)

	("mam"       "mie" 0 CT 1 LP)
	("masz"      "mie" 0 CT 2 LP)
	("ma"        "mie" 0 CT 3 LP)
	("mamy"      "mie" 0 CT 1 LM)
	("macie"     "mie" 0 CT 2 LM)
	("maj"      "mie" 0 CT 3 LM)

	("wiem"      "wiedzie" 0 CT 1 LP)
	("wiesz"     "wiedzie" 0 CT 2 LP)
	("wie"	     "wiedzie" 0 CT 3 LP)
	("wiemy"     "wiedzie" 0 CT 1 LM)
	("wiecie"    "wiedzie" 0 CT 2 LM)
	("wiedz"    "wiedzie" 0 CT 3 LM)

	("jem"       "je" 0 CT 1 LP) 
	("jesz"	     "je" 0 CT 2 LP)
	("je"	     "je" 0 CT 3 LP)
	("jemy"	     "je" 0 CT 1 LM)
	("jedzicie"  "je" 0 CT 2 LM)
	("jedz"     "je" 0 CT 3 LM)
	
	("dam"       "da" 0 CT 1 LP)
	("dasz"	     "da" 0 CT 2 LP)
	("da"	     "da" 0 CT 3 LP)
	("damy"	     "da" 0 CT 1 LM)
	("dacie"     "da" 0 CT 2 LM)
	("dadz"     "da" 0 CT 3 LM)))

;; Szuka w  VERB-PRESENT czasownika o temacie ROOT i koniugacji CONJ	
(defun dict-verb-present-lookup (root conj)
  (dolist (p verb-present)
    (if (and (string-equal root (car p))
	     (= conj (nth 2 p)))
	(return (nth 1 p)))))

;; Szuka w  VERB-PRESENT czasownika o bezokoliczniku INF i koniugacji CONJ	
(defun dict-verb-infinitive-lookup (inf conj)
  (dolist (p verb-present)
    (if (and (string-equal inf (nth 1 p))
	     (= conj (nth 2 p)))
	(return (nth 1 p)))))

;; Szuka w VERB-PRESENT czasownika o bezokoliczniku INF. Zwraca
;; (cons BEZOKOLICZNIK KONIUGACJA)
(defun dict-verb-search-infinitive (inf)
  (dolist (p verb-present)
    (if (string-equal inf (nth 1 p))
	(return (cdr p)))))

;; Szuka czasownika VERB w tabeli czasownikw koniugacji archaicznej
(defun dict-verb-individual-lookup (verb)
  (dolist (x verb-individual)
    (when (string-equal (car x) verb)
      (return (cdr x)))))

(defun maybe-infinitive-p (verb)
  (or (match-ending verb "")
      (and (match-ending verb "c") (vocal-ending (remove-ending verb 1)))))

;; VDLIST jest list moliwych deskryptorw czasownika. VERB jest jego form osobow,
;; znalezion w tekcie. Jeli VDLIST skada si z kilku elementw, pytamy uytkownika
;; ktry z nich jest prawidowym. Moe by on wie lepiej ... :) Odpowiedzi dodajemy do
;; sownika VERB-PRESENT.
(defun learn-verb (vdlist verb)
  (cond
   ((not (null vdlist))
    (while t
      (let ((ans
	     (get-list-answer
	      "Ktra z odpowiedzi jest prawidowa (numer, lista numerw, <CR> jeli adna)?")))

	(cond
	 ((listp ans)
	  (unless (dolist (x ans)
		    (cond
		     ((not (numberp x))
		      (format t "Nie liczba: ~A. Sprbuj jeszcze raz.~%" x)
		      (return t))
		     ((>= x (length vdlist))
		      (format t "Nie ma odpowiedzi ~A~%" x)
		      (return t))))
	    (dolist (n ans)
	      (let ((elem (nth n vdlist)))
		(let ((infinitive-variants (nthcdr 5 elem)))
		  (if (= (length infinitive-variants) 1)
		      (dict-store verb-present
				  (list (verb-get-root elem)
					(car infinitive-variants)
					(verb-get-conj elem)))
		    (let ((i 0))
		      (dolist (inf infinitive-variants)
			(format t "~A: ~A~%" i inf)
			(setq i (1+ i)))
		      (let ((i (getnumber "Ktry z bezokolicznikw?")))
			(dict-store verb-present
				    (list (verb-get-root elem)
					  (nth i infinitive-variants)
					  (verb-get-conj elem))))))))))
	  (return))

	 ((or (not ans) (and (stringp ans) (string= ans "")))
	  (when (getyn "Nowa definicja  (Tak/Nie)?")
	    (format t "| Definicja formy:~%")
	    (let ((person (verb-get-person (car vdlist)))
		  (number (verb-get-number (car vdlist)))
		  (tense (while t
			   (let ((tn (getsymbol "| Czas:")))
			     (if (member tn '(ct cprze))
				 (return tn)
			       (format t "| BD: Niepoprawny czas.~%"))))))
	      (setq root (getstring "| Temat:"))
	      (setq inf (getstring "| Bezokolicznik:"))
	      (setq conj (getnumber "| Konjugacja:" 1 3))
	      (let ((dfn (list inf conj tense person number)))
	      	(format t "|| Definicja: ")
		(print-result dfn)
		(when (getyn "|| Doda do sownika?")
		  (dict-store verb-present (list root inf conj))))))
	  
	  (return))
	 
	 (t
	  (format t "Niepoprawna odpowied.~%"))))))
   ((maybe-infinitive-p verb)
    (format t "~A: " verb)
    (when (getyn "bezokolicznik?")
      (let ((conj (getnumber "| Konjugacja:" 1 3))
	    (root (getstring "| Temat:"))
	    (ja (getstring "| 1 os. lp. cz.t. (ja):")))
	(let ((dfn (list verb conj 'CT 1 'LP)))
	  (format t "|| Definicja: ")
	  (print-result dfn)
	  (when (getyn "|| Doda do sownika?")
	    (dict-store verb-present (list root verb conj)))))))))

(defun get-verb (word)
  (mapcan
   #'(lambda (x)
       (when (or (= (length x) 2) (= (length x) 5))
	   (list x)))
   (analyze-verb word)))

(defun analyze-and-learn-verb (word)
  (let ((r (analyze-verb word))
	(variants nil))
    (format t "Analiza 'a priori':~%")
    (let ((n 0))
      (mapcar #'(lambda (x)
		  (format t "~A: " n)
		  (setq n (1+ n))
		  (print-result x)
		  (if (> (length x) 5)
		      (setq variants t)))
	      r))
    (when (or variants (maybe-infinitive-p word))
      (learn-verb r word))))

;; Klasyka lispa... Czytamy form czasownika, analizujemy i wywetlamy wyniki. Jeli
;; nie dajemy sobie rady, no c, pytamy uytkownika...
(defun read-print-verb ()
  (format t "~&Czasownik: ")
  (let ((line (read-line)))
    (when (and line (string-not-equal line ""))
      (analyze-and-learn-verb line)
      t)))
	
(defun check-verbs ()
  (while (read-print-verb)))


;; Sprawdzanie.
;(load "util.lisp")
;(load-dictionaries)
;(check-verbs)
;(save-dictionaries)
