;; Analiza przymiotnikw
;; Nadaje si rwnie do analizy imiesoww odmiennych.

;; Skrty:
;;  RM  -- rodzaj mski
;;  R  -- rodzaj eski
;;  RN  -- rodzaj nijaki
;;
;; Definicja formy
;;
;;  '(forma-sownikowa rodzaj przypadek liczba)

(defvar adjective-drivers)
(setq adjective-drivers
      (list
       ;; ma-y -- M. lub B. rm. lp.
       ;; drodz-y -- M. lm. formy mskoosobowej
       (list "y"
	     #'(lambda (word root ending-length)
		 (list word 'RM '(M B) 'LP))
	     #'(lambda (word root ending-length)
		 (unless (or (match-ending word "y")
			     (match-ending word "ny"))
		   (list (deduce-ma-nominative word) 'RM 'M 'LM))))
       
       ;; ostatnie -- M. lub B. rn. lp. oraz wszystkich rodzajw lm.
       ;; w formie niemskoosobowej
       (list "ie"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") 'RN '(M B) 'LP))
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") '(RM R RN) '(M B) 'LM)))
       
       ;; ma-e -- M. lub B. rn. lp. oraz wszystkich rodzajw lm. w
       ;; formie niemskoosobowej
       (list "e"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") 'RN '(M B) 'LP))
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") '(RM R RN) '(M B) 'LM)))
       
       ;; ostatn-imi  -- N. lm.
       (list "imi"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") '(RM R RN) 'N 'LM)))
       
       ;; ma-ymi     -- N. lm.
       (list "ymi"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") '(RM R RN) 'N 'LM)))

       ;; ostatn-i -- M. i B. rm. lp.
       ;; mal-i    -- M. rm. lm. (forma mskoosobowa)
       (list "i"
	     #'(lambda (word root ending-length)
		 (list word 'RM '(M B) 'LP))
	     #'(lambda (word root ending-length)
		 (list (deduce-ma-nominative word) '(RM R RN) 'M 'LM)))
	     
       ;; ostatn-iego -- D. i B. lp. rm.
       (list "iego"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") 'RM '(D B) 'LP)))
       
       ;; ma-ego     -- D. i B. lp. rm. i rn.
       (list "ego"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") 'RM '(D B) 'LP)))

       ;; ostatn-iemu -- C. lp. rm.
       (list "iemu"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") 'RM 'C 'LP)))
       
       ;; ma-emu     -- C. lp. rm.
       (list "emu"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") 'RM 'C 'LP)))

       ;; ostatn-im   -- N. i Mc. lp. rm. i rn. oraz C. wszystkich rodzajw lm.
       (list "im"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") '(RM RN) '(N Mc) 'LP))
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") '(RM R RN) 'C 'LM)))
       
       ;; ma-ym      -- N. i Mc. lp. rm. i rn. oraz C. rm. lm.
       (list "ym"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") '(RM RN) '(N Mc) 'LP))
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") '(RM R RN) 'C 'LM)))

       ;; ostatn-ia -- M. r. lp.
       (list "ia"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") 'R 'M 'LP)))
       
       ;; ma-a -- M. r. lp.
       (list "a"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") 'R 'M 'LP)))

       ;; ostatn-iej  -- D.,C.,Mc. r. lp.
       (list "iej"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") 'R '(D C Mc) 'LP)))
       
       ;; ma-ej      -- D.,C.,Mc. r. lp.
       (list "ej"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") 'R '(D C Mc) 'LP)))
       
       ;; ostatn-i   -- B.,N. r. lm.
       (list "i"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") 'R '(B N) 'LP)))
       
       ;; ma-       -- B.,N. r. lm.
       (list ""
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") 'R '(B N) 'LP)))

       ;; ostatn-ich  -- D.,B.,Mc. lm.
       (list "ich"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "i") '(RM R RN) '(D B Mc) 'LM)))
       
       ;; ma-ych     -- D.,B.,Mc. lm.
       (list "ych"
	     #'(lambda (word root ending-length)
		 (list (string-concat root "y") '(RM R RN) '(D B Mc) 'LM)))))

;; deduce-ma-nominative = deduce nominative from a masculine-animate root
;; WORD jest form mianownika liczby mskoosobowej.
;; Funkcja zwraca list moliwych form sownikowych podanego przymiotnika,
;; uwzgldniajc zachodzce wymiany spgosek (palatalizacj) wg. poniszej
;; tabeli oraz wymian samogosek o - e w przymiotnikach na -ony i przymiotniku
;; "wesoy".
;;
;; +----------------------------+--------------------------+
;; |          Kocwka		|    Przykad 		   |
;; | osnowy M.lp.| osnowy M. lm.|                          |
;; +-------------+--------------+--------------------------+
;; | g           |  dz		|    drogi   - drodzy	   |
;; | k           |  c		|    wysoki  - wysocy	   |
;; | r           |  rz		|    mdry   - mdrzy	   |
;; | d           |  d		|    mody   - modzi	   |
;; |            |             |    duy    - duzi        |
;; | ch          |  		|    kruchy  - krusi	   |
;; | sz          |  		|    starszy - starsi	   |
;; | s           |  		|	ysy - ysi	   |
;; |            |  		|       duy - duzi	   |
;; |            |  l		|       may - mali	   |
;; | t           |  		| wymienity - wymienici  |
;; | st          |  		| siarczasty - siarczaci  |
;; | s          |  l		|   doniosy - donioli    |
;; | z          |  l		|    zmarzy - zmarli	   |
;; +----------------------------+--------------------------+
;;  
(defun deduce-ma-nominative (word)
  (cond
   ((match-ending word "dzy")
    (string-concat (remove-ending word 3) "gi"))
   ((match-ending word "cy")
    (string-concat (remove-ending word 2) "ki"))
   ((match-ending word "rzy")
    (string-concat (remove-ending word 3) "ry"))
   ((match-ending word "si")
    (let ((r (remove-ending word 2)))
      (list
       (string-concat r "chy")
       (string-concat r "szy")
       (string-concat r "sy"))))
   ((match-ending word "dzi")
    (string-concat (remove-ending word 2) "y"))
   ((match-ending word "zi")
    (string-concat (remove-ending word 2) "y"))
   ((match-ending word "li")
    (let ((r (remove-ending word 2)))
      ; ``wesoy - weseli'' wydaje si by jedynym wyjtkiem we
      ; wspczesnej polszczynie. Uywamy match-ending zamiast string-equal,
      ; eby uwzgldni ewentualne przyrostki (niewesoy, przewesoy itp.)
      (if (match-ending r "wese")
	  (string-concat (remove-ending r 4) "wesoy")
	(string-concat r "y"))))
   ((match-ending word "li")
    (string-concat (remove-ending word 3) "sy"))
   ((match-ending word "li")
    (string-concat (remove-ending word 3) "zy"))
   ((match-ending word "ci")
    (string-concat (remove-ending word 3) "sty"))
   ((match-ending word "ci")
    (string-concat (remove-ending word 2) "ty"))
   ((match-ending word "eni")
    (string-concat (remove-ending word 3) "ony"))
   ((match-ending word "ni")
    word)
   ((match-ending word "ki")
    word)
   ((match-ending word "gi")
    word)
   (t
    (string-concat (remove-ending word 1) "y"))))

(defun analyze-adjective (word)
  (mapcan #'(lambda (x)
	      (burst-digest x 0))
	  (analyze-word word adjective-drivers)))


;;; Sownik

(defvar adjective-dictionary nil)

(defun dict-lookup-adjective (word)
  (member word adjective-dictionary :test #'string-equal))

(defun lookup-adjective (vlist)
  (mapcan
   #'(lambda (v)
       (if (dict-lookup-adjective (adj-get-dictform v))
	   (list v)))
   vlist))


(defun learn-adjective (flist orig)
  (when (not (null flist))
    (while t
      (let ((ans
	     (get-list-answer
	      "Ktra z odpowiedzi jest prawidowa (numer, lista numerw, <CR> jeli adna)?")))
	(cond
	 ((listp ans)
	  (unless (dolist (x ans)
		    (cond
		     ((not (numberp x))
		      (format t "Nie liczba: ~A. Sprbuj jeszcze raz.~%" x)
		      (return t))
		     ((>= x (length flist))
		      (format t "Nie ma odpowiedzi ~A~%" x)
		      (return t))))
	    (mapcar
	     #'(lambda (n)
		 (let ((elem (nth n flist)))
		   (dict-store adjective-dictionary (adj-get-dictform elem))))
	     ans)
	    (return)))
	 ((or (not ans) (and (stringp ans) (string= ans "")))
	  (return))
	 (t
	  (format t "Niepoprawna odpowied.~%")))))) )

(defun get-adjective (word)
  (let ((r (analyze-adjective word)))
    (lookup-adjective r)))

(defun analyze-and-learn-adjective (word)
  (let ((r (analyze-adjective word)))
    (format t "Analiza 'a priori':~%")
    (let ((n 0))
      (mapcar #'(lambda (x)
		  (format t "~A: " n)
		  (setq n (1+ n))
		  (print-result x))
	      r))
    
    (let ((res (lookup-adjective r)))
      (cond
       (res
	(format t "Analiza a posteriori:~%")
	(mapcar #'(lambda (x)
		    (print-result x))
		res))
       (t
	(learn-adjective r word))))))

(defun read-print-adjective ()
  (format t "~&Przymiotnik: ")
  (let ((line (read-line)))
    (when (and line (string-not-equal line ""))
      (analyze-and-learn-adjective line)
      t)))

(defun check-adjectives ()
  (while (read-print-adjective)))

;; Sprawdzanie
;(load "util.lisp")
;(load-dictionaries)
;(check-adjectives)
;(save-dictionaries)
;(quit)
		
